/**
 * Copyright (c) 2010, Stefan Langer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Element34 nor the names of its contributors may
 *       be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS ROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import scala.collection.mutable.ListBuffer
import scala.collection.mutable.Stack
import scala.collection.mutable.Set
import scala.xml._

import sbt._

object Utils {
  
	/**
	 * Return all (transitive) dependencies of the given project. The given 
	 * project is not included in the returned sequence. 
	 */
	def getAllDependencies(project: Project): Seq[Project] = {
		val projects = new ListBuffer[Project]()
		val visited = Set[String]()
		val stack = new Stack[Project]()
  
		for (dep <- project.dependencies) {
			stack += dep
		}
		while (!stack.isEmpty) {
			val current = stack.pop
			
			if (!visited.contains(current.name)) {
				projects += current
				visited + current.name
			}
   
	   		for (dep <- current.dependencies) {
				stack += dep
			}
		}
  
		projects.toList
	}
  
	def toEclipsify(project: Project): Eclipsify = project.asInstanceOf[Eclipsify]

	def writeNodeSeq(body: Eclipsify => NodeSeq)(implicit project: Project): NodeSeq = {
		writeNodeSeq(true)(body)(project)
	}

	def writeNodeSeq(predicate: => Boolean)(body: Eclipsify => NodeSeq)(implicit project: Project): NodeSeq = {
		if(predicate) {
			body(toEclipsify(project))
		}
		else NodeSeq.Empty
	}

	def get[T](body: Eclipsify => Environment#Property[T])(implicit project: Project): T = {
		body(toEclipsify(project)).value
	}
}




